"""CLI Input params."""

from redbrick.cli.input.api_key import CLIInputAPIKey
from redbrick.cli.input.number import CLIInputNumber
from redbrick.cli.input.profile import CLIInputProfile
from redbrick.cli.input.select import CLIInputSelect
from redbrick.cli.input.text import CLIInputText
from redbrick.cli.input.url import CLIInputURL
from redbrick.cli.input.uuid import CLIInputUUID
